#!/usr/bin/env python

import threading
import wx
from rt import ut
from argmnt import opt
import wfrm

# l, m, r
btn_downs = ( wx.EVT_LEFT_DOWN.typeId, wx.EVT_MIDDLE_DOWN.typeId, wx.EVT_RIGHT_DOWN.typeId )
btn_ups = ( wx.EVT_LEFT_UP.typeId, wx.EVT_MIDDLE_UP.typeId, wx.EVT_RIGHT_UP.typeId )

def new_dclick():
	e = ut.Empty()
	e.timer = None
	e.pos = None
	e.i = -1
	
	def cancel():
		e.i = -1
		if e.timer:
			e.timer.cancel()
			e.timer = None
	def check(event):
		typeId = event.GetEventType()
		pos = event.GetPosition()

		if not e.timer:
			if typeId in btn_ups:
				e.i = btn_ups.index(typeId)
				e.pos = pos
				e.timer = threading.Timer(0.3, cancel)
				e.timer.start()
			return ''

		i = e.i
		if typeId != btn_downs[i]:
			cancel()
		if typeId == btn_ups[i]:
			return 'lmr'[i]
		return ''
	e.check = check
	return e

class MyFrame(wx.Frame):
	def __init__(self, *args, **kwds):
		kwds["style"] = wx.DEFAULT_FRAME_STYLE
		wx.Frame.__init__(self, *args, **kwds)

		self.SetSize((320,240))
		self.panel = wx.Panel(self, wx.ID_ANY)
		sizer_1 = wx.BoxSizer(wx.VERTICAL)
		sizer_1.Add(self.panel, 1, wx.EXPAND, 0)
		self.SetSizer(sizer_1)
		self.Layout()

		evs = (
			wx.EVT_MOTION,
			wx.EVT_LEFT_DOWN,
			wx.EVT_LEFT_UP,
			wx.EVT_MIDDLE_DOWN,
			wx.EVT_MIDDLE_UP,
			wx.EVT_RIGHT_DOWN,
			wx.EVT_RIGHT_UP,
		)
		for ev in evs:
			self.panel.Bind(ev, self.hdr)
		self.panel.Bind(wx.EVT_PAINT, self.OnPaint)
		self.Bind(wx.EVT_CLOSE, self.OnClose)

		fn_yaml = opt.get('-fn', 'cube.yaml')
		srv = opt.get('-srv')
		kill_all = opt.index('-ka') >= 0
		self.wf = wfrm.new( fn_yaml, srv=srv, kill_all=kill_all )
		self.dclick = new_dclick()

	def OnPaint(self, event):
		(w, h) = self.panel.GetSize()
		lst = self.wf.get_lst(w, h)
		dc = wx.PaintDC(self.panel)
		for (x1, y1, x2, y2) in lst:
			dc.DrawLine(x1, y1, x2, y2)

	def hdr(self, event):
		dck = self.dclick.check(event)
		if dck == 'r':
			self.wf.turn_view()
		elif dck == 'l':
			self.wf.reset()
		if dck:
			self.panel.Refresh()
			return

		typeId = event.GetEventType()
		(x, y) = event.GetPosition()
		if typeId in btn_downs:
			mode = ('rota', 'move', 'zoom')[ btn_downs.index(typeId) ]
			self.wf.down(x, y, mode)
		elif typeId in btn_ups:
			self.wf.up()
		elif typeId == wx.EVT_MOTION.typeId and self.wf.in_drag():
			(w, h) = self.panel.GetSize()
			self.wf.drag(x, y, w, h)
			self.panel.Refresh()

	def OnClose(self, event):
		self.wf.quit()
		self.Destroy()

class MyApp(wx.App):
	def OnInit(self):
		frame_1 = MyFrame(None, wx.ID_ANY, "")
		self.SetTopWindow(frame_1)
		frame_1.Show()
		return 1

if __name__ == "__main__":
	app = MyApp(0)
	app.MainLoop()
# EOF
